import * as fs from 'node:fs';
import * as path from 'node:path';
import stringHash from 'string-hash';
import { $css } from 'taddy';
import { getCacheDir } from './config.js';

const LAST_INDEX = 0;
const STYLES = [];
function getStylesState() {
    const { rules } = $css.ruleInjector.styleSheet;
    const newRules = rules.slice(LAST_INDEX);
    // there is also "appending" rules, that don't change the index, but change the rule
    // TODO: think how to handle that
    // LAST_INDEX = rules.length;
    const added = [...newRules].map((rule) => rule.cssText || '');
    STYLES.push(...added);
    return {
        added,
    };
}
function getMtime(filepath) {
    return Number(fs.statSync?.(filepath).mtime);
}
const filesMap = new Map();
const contentMap = new Map();
function readFileSync(filepath) {
    if (!fs.existsSync(filepath))
        return '';
    let data = '';
    try {
        const mtime = getMtime(filepath);
        if (filesMap.has(mtime)) {
            return filesMap.get(mtime);
        }
        data = fs.readFileSync(filepath).toString();
        if (!data)
            return '';
        filesMap.set(mtime, data);
        contentMap.set(data, mtime);
    }
    catch (e) {
        console.error('error', e);
    }
    return data;
}
function mkdir(filepath) {
    fs.mkdirSync(path.dirname(filepath), { recursive: true });
}
function appendFile(filepath, append) {
    // const code = content + append;
    // if (contentMap.has(code)) {
    //     return;
    // }
    // fs.appendFile(filepath, append, () => {});
    mkdir(filepath);
    fs.appendFileSync(filepath, append);
}
function writeFile(filepath, code) {
    // const code = content + append;
    // if (contentMap.has(code)) {
    //     return;
    // }
    // fs.appendFile(filepath, append, () => {});
    mkdir(filepath);
    fs.writeFileSync(filepath, code);
}
/** don't merge declarations in plugin, currently considering only dev mode */
$css.ruleInjector.styleSheet.options.mergeDeclarations = false;
class Output {
    env;
    config;
    filepath;
    cacheDir;
    constructor({ env, config }) {
        this.env = env;
        this.cacheDir = getCacheDir();
        this.config = Object.assign({
            // getCSSFilename = (content) => `atoms-${stringHash(content)}.css`,
            cssFilename: () => `taddy.css`,
            cssFilepath: (filename) => path.join(this.cacheDir, filename),
        }, config);
        const filename = this.resolveFilepath(this.config.cssFilename);
        this.filepath = this.resolveFilepath(this.config.cssFilepath, filename);
        mkdir(this.filepath);
        // console.log('filepath', this.filepath);
        /* clean cache */
        // fs.writeFileSync(this.filepath, '');
    }
    resolveFilepath(getter, ...params) {
        if (typeof getter !== 'string') {
            // @ts-expect-error TODO: fix getter params
            return getter(...params);
        }
        return getter;
    }
    save({ sourceMap, filenameRelative, target, }) {
        const stylesData = readFileSync(this.filepath);
        const { added } = getStylesState();
        // console.log({added});
        // TODO: improve filter performance
        const diffStyles = added
            .filter((x) => !stylesData.includes(x))
            .join('');
        appendFile(this.filepath, diffStyles);
        // appendFile(this.filepath, sourceMap);
        const localFilename = stringHash(`${filenameRelative}`) + '.taddy';
        const localFilepath = path.join(this.cacheDir, 
        // stringHash(`${filename}:${added}`) + '.taddy.css',
        localFilename);
        writeFile(localFilepath + '.css', added.join('').replace(/}$/, sourceMap + '}'));
        if (target === 'remix') {
            writeFile(localFilepath + '.cjs', `require('./${localFilename}.css');`);
            writeFile(localFilepath + '.js', `import './${localFilename}.css'`);
            return { importName: `@taddy/babel-plugin/cache/${localFilename}` };
        }
        return { importName: `@taddy/babel-plugin/cache/${localFilename}.css` };
    }
}

export { Output };
//# sourceMappingURL=Output.js.map
