'use strict';

var evaluatePath = require('babel-helper-evaluate-path');
var resolve = require('resolve');
var path = require('node:path');
var taddy = require('taddy');
var config = require('../config.cjs');
var utils = require('./utils.cjs');
var findBindings = require('./findBindings.cjs');
var buildCodeByPath = require('./buildCodeByPath.cjs');

function _interopNamespaceDefault(e) {
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var taddy__namespace = /*#__PURE__*/_interopNamespaceDefault(taddy);

// import {transformSync} from '@babel/core';
// import tsPreset from '@babel/preset-typescript';
// import reactPreset from '@babel/preset-react';
// import envPreset from '@babel/preset-env';
// const DEFAULT_PRESETS = [
//     [tsPreset, {allExtensions: true, isTSX: true}],
//     // '@babel/preset-react',
//     [
//         envPreset,
//         {
//             targets: {node: '12'},
//             useBuiltIns: false,
//             ignoreBrowserslistConfig: true,
//         },
//     ],
// ];
const rpc = require('sync-rpc');
const client = rpc(__dirname + '/transformWorker.js', 'Evaluate');
const EXTENSIONS = ['.es6', '.es', '.tsx', '.ts', '.jsx', '.js', '.mjs'];
const macroRe = new RegExp(config.MACRO_NAME.replace('.', '\\.'), 'g');
// webpack "require" critical dependency issue workaround
const nodeRequire = new Function('require', `return typeof require !== 'undefined'
        ? require
        : (typeof globalThis !== 'undefined' ? globalThis : global).require; `)(module.require);
function evaluate(currentPath, { exec = true }) {
    const result = evaluatePath(currentPath);
    if (result.confident) {
        findBindings.findBindings(currentPath);
        return { value: result.value };
    }
    if (!exec)
        return { error: new Error('EXEC_REQUIRED') };
    let content = '';
    let code = '';
    try {
        const callbackName = '__taddy__';
        content = buildCodeByPath.buildCodeByPath(currentPath)
            // hack to avoid processing by babel-macro
            .replace(macroRe, config.PACKAGE_NAME)
            .concat(`\n\n;${callbackName}(${currentPath.toString()})`);
        // TODO: improve Hub type
        const { opts } = currentPath.hub.file;
        const ext = path.extname(opts.filename);
        const basename = path.basename(opts.filename, ext);
        const dirname = path.dirname(opts.filename);
        const filename = path.join(dirname, basename + utils.EVAL_FILENAME_POSTFIX + ext);
        // const options = {
        //     babelrc: false,
        //     configFile: false,
        //     filename,
        //     plugins: [
        //         /*...opts.plugins*/
        //     ],
        //     presets: [/*...opts.presets*/ ...DEFAULT_PRESETS],
        // };
        ({ code } = client({ content, filename }) || {});
        if (!code)
            return { error: new Error('TRANSPILATION_ERROR') };
        const exec = new Function('require', callbackName, code);
        let value;
        exec((filepath) => {
            if (filepath === config.PACKAGE_NAME)
                return taddy__namespace;
            const requirePath = resolve.sync(filepath, {
                extensions: EXTENSIONS,
                basedir: path.dirname(opts.filename),
            });
            return nodeRequire(requirePath);
        }, (result) => {
            value = result;
        });
        return { value };
    }
    catch (error) {
        // console.log('evaluate error', {content, code, error});
        return { error };
    }
}

exports.evaluate = evaluate;
//# sourceMappingURL=evaluate.cjs.map
