import { getEnv, PACKAGE_NAME } from './config.js';
import { isTaddyEvaluation } from './helpers/utils.js';
import { macro } from './macro-plugin.js';

function getImportNames(path) {
    const localName = path.node.local.name;
    if (path.isImportSpecifier()) {
        const imported = path.node.imported;
        return {
            localName,
            importedName: imported.name,
        };
    }
    if (path.isImportDefaultSpecifier()) {
        return { localName, importedName: 'default' };
    }
    return { localName, importedName: 'namespace' };
}
function findReferences(paths) {
    const references = {};
    for (const path of paths) {
        const { localName, importedName } = getImportNames(path);
        const binding = path.scope.getBinding(localName);
        if (binding?.referencePaths) {
            references[importedName] = binding.referencePaths;
        }
    }
    return references;
}
function plugin(babel, options) {
    const env = getEnv(babel);
    return {
        name: '@taddy/babel-plugin',
        pre() {
            this.references = {};
        },
        visitor: {
            // TODO: support require expression
            ImportDeclaration(path, state) {
                // console.log('run', state.file.code, state.filename);
                if (isTaddyEvaluation(state)) {
                    return;
                }
                if (path.node.source.value !== PACKAGE_NAME) {
                    return;
                }
                Object.assign(this.references, findReferences(path.get('specifiers')));
                if (!(this.references.css || this.references.mixin)) {
                    return;
                }
                macro({
                    references: this.references,
                    babel,
                    state,
                    config: {
                        env,
                        ...options,
                    },
                });
            },
        },
    };
}

export { plugin as default };
//# sourceMappingURL=plugin.js.map
